unit Unit1;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, HKStreamCol, ExtCtrls;

type
  TfrmMain = class(TForm)
    btnLoad: TButton;
    HKStreams1: THKStreams;
    Memo1: TMemo;
    ListBox1: TListBox;
    btnSave: TButton;
    SaveDialog1: TSaveDialog;
    OpenDialog1: TOpenDialog;
    GroupBox1: TGroupBox;
    Edit1: TEdit;
    Edit2: TEdit;
    Edit3: TEdit;
    Edit4: TEdit;
    CheckBox1: TCheckBox;
    CheckBox2: TCheckBox;
    ComboBox1: TComboBox;
    Panel1: TPanel;
    Label1: TLabel;
    Label2: TLabel;
    Panel2: TPanel;
    procedure btnSaveClick(Sender: TObject);
    procedure btnLoadClick(Sender: TObject);
  private
    { Private declarations }
    function SaveHKFile(FileName: TFileName): boolean;
    function LoadHKFile(FileName: TFileName): boolean;
    procedure GetSettings(var lstSettings: TStringList; AParent: TComponent);
    procedure SetSettings(lstSettings: TStringList);
  public
    { Public declarations }
  end;

var
 frmMain: TFrmMain;
   
implementation

{$R *.dfm}

//--------------------------------------------------------
// Get Text, Checked-State of controls on a Control (Groupbox)
procedure TfrmMain.GetSettings(var lstSettings: TStringList; AParent: TComponent);
//--------------------------------------------------------
var
  i: Integer;
  AControl: TControl;
begin
  for i := 0 to ComponentCount - 1 do
  begin
    // Only TControls
    if not (Components[i] is TControl) then Continue;
    AControl := TControl(Components[I]);
    // Only Controls which belong to AParent (ie Groupbox1)
    if AControl.Parent <> AParent then Continue;
    // TEdit, TMemo,...
    //-----------------------------
    if AControl is TCustomEdit then
      lstSettings.Add(TCustomEdit(AControl).Name + #9 + TCustomEdit(AControl).Text);
    // TComboBox
    //-----------------------------
    if AControl is TComboBox then
      lstSettings.Add(TComboBox(AControl).Name + #9 + TComboBox(AControl).Text);
    // TCheckBox
    //-----------------------------
    if AControl is TCheckBox then
      if TCheckBox(AControl).Checked = True then
        lstSettings.Add(TCheckBox(AControl).Name + #9 + 'True')
      else
        lstSettings.Add(TCheckBox(AControl).Name + #9 + 'False');
  end;
end;

//--------------------------------------------------------
// Set Text, Checked-State of controls on a Control (Groupbox)
procedure TfrmMain.SetSettings(lstSettings: TStringList);
//--------------------------------------------------------
var
  i, iTemp: Integer;
  AControl: TComponent;
  CompName, CompProp: string;

  function StrToBool(AString: string): Boolean;
  begin
    Result := AString = 'True';
  end;
  
begin
  for i := 0 to lstSettings.Count - 1 do
  begin
    // Parse the Component name
    CompName := Copy(lstSettings[i], 1, Pos(#9, lstSettings[i]) - 1);
    // Parse the Component property (Text, Checked state...)
    iTemp := Pos(#9, lstSettings[i]);
    CompProp := Copy(lstSettings[i], iTemp + 1, Length(lstSettings[i]) - iTemp);
    // Find the control
    AControl := FindComponent(CompName);
    if AControl <> nil then
    begin
      // TCheckBox
      //-----------
      if (CompProp = 'True') or (CompProp = 'False') then
      begin
        // If TCustomEdit(AControl).Parent <> AParent then Continue;
        TCheckBox(AControl).Checked := StrToBool(CompProp)
      end
      else
      // Other Components
      //------------------
      begin
          // TComboBox
          //-----------
          if AControl is TComboBox then
          begin
            iTemp := TComboBox(AControl).Items.IndexOf(CompProp);
            if iTemp = -1 then iTemp := 0;
            TComboBox(AControl).ItemIndex := iTemp;
          end
          else
          // TCustomEdit
          //------------
          begin
            if AControl is TCustomEdit then
              TCustomEdit(AControl).Text := CompProp;
          end;
      end;
    end; {AControl <> nil}
  end; {end for}
end;

//--------------------------------------------------------
// Save our File
function TfrmMain.SaveHKFile(FileName: TFileName): Boolean;
//--------------------------------------------------------
var
  ms: TMemoryStream;
  i: integer;
  s: string;
  StrList: TStringList;
begin
  ms := TMemoryStream.Create;
  try
    with HKStreams1 do
    begin
      // Version Info
      //--------------------------------
      i := 1;
      ms.Write(I, SizeOf(I));
      AddStream('Version-Info', ms);
      ms.Clear;
      // Author Info
      //--------------------------------
      s := 'Your Name';
      ms.WriteBuffer(s[1], Length(s));
      AddStream('Author-Info', ms);
      ms.Clear;
      // Memo1
      //--------------------------------
      Memo1.Lines.SaveToStream(ms);
      AddStream('Memo1', ms);
      ms.Clear;
      // ListBox1
      //-------------------------------
      ListBox1.Items.SaveToStream(ms);
      AddStream('ListBox1', ms);
      ms.Clear;
      // Different Controls on Groupbox1
      //-------------------------------
      StrList := TStringList.Create;
      try
        GetSettings(StrList, GroupBox1);
        StrList.SaveToStream(ms);
      finally
        StrList.Free;
      end;
      AddStream('Settings', ms);
      ms.Clear;
      // Save everything
      // -----------------------------
      SaveToFile(FileName);
      ClearStreams;
    end; {with}
  finally
    ms.Free;
  end;
end;

//--------------------------------------------------------
// Load our File
function TfrmMain.LoadHKFile(FileName: TFileName): Boolean;
//--------------------------------------------------------
var
  ms: TMemoryStream;
  i: Integer;
  s: string;
  StrList: TStringList;
begin
  if not FileExists(FileName) then
  begin
    ShowMessage('File not found');
    // Customize Error handling...
    Exit;
  end;
  try
    HKStreams1.LoadFromFile(FileName);
  except
    on ECorruptFile do Exit;
    on EStreamError do Exit;
  end;
  ms := TMemoryStream.Create;
  try
    with HKStreams1 do
    begin
      // Version Info
      //--------------------------------
      GetStream('Version-Info', ms);
      ms.read(i, SizeOf(i));
      label1.Caption := 'Version: ' + IntToStr(i);
      ms.Clear;
      // Author Info
      //--------------------------------
      GetStream('Author-Info', ms);
      SetLength(S, ms.Size);
      ms.read(S[1], ms.Size);
      Label2.Caption := 'Author: ' + s;
      ms.Clear;
      // Memo1
      //--------------------------------
      GetStream('Memo1', ms);
      Memo1.Lines.LoadFromStream(ms);
      ms.Clear;
      // ListBox1
      //-------------------------------
      GetStream('ListBox1', ms);
      ListBox1.Items.LoadFromStream(ms);
      ms.Clear;
      // Different Controls on Groupbox1
      //-------------------------------
      GetStream('Settings', ms);
      StrList := TStringList.Create;
      try
        StrList.LoadFromStream(ms);
        SetSettings(StrList);
      finally
        StrList.Free;
      end;
      { ----------------------------- }
      ClearStreams;
    end; {with}
  finally
    ms.Free;
  end;
end;

//--------------------------------------------------------
// Save Dialog
procedure TfrmMain.btnSaveClick(Sender: TObject);
//--------------------------------------------------------
begin
  if SaveDialog1.Execute then
  begin
    SaveHKFile(SaveDialog1.FileName);
  end;
  // Clear controls
  ListBox1.Clear;
  Memo1.Clear;
  Edit1.Clear;
  Edit2.Clear;
  Edit3.Clear;
  Edit4.Clear;
  CheckBox1.Checked := not CheckBox1.Checked;
  CheckBox2.Checked := not CheckBox2.Checked;
end;

//--------------------------------------------------------
// Open Dialog
procedure TfrmMain.btnLoadClick(Sender: TObject);
//--------------------------------------------------------
begin
  if OpenDialog1.Execute then
  begin
    LoadHKFile(OpenDialog1.FileName);
  end;
end;

end.
